$Title  Dynamic TD/BU hybrid model - MPSGE Code 

set	taup		Time periods including post-terminal /2005*2101/
	tau(taup)	Time periods /2005*2100/
	tau0(taup)	First time period
	tauL(taup)	Last time period
	tauPT(taup)	Post-terminal time period;

tau0(tau)	= yes$(ord(tau) = 1);
tauL(tau)	= yes$(ord(tau) = card(tau));
tauPT(taup)	= yes$(ord(taup) = card(taup));

parameter	year(tau)	Year as a numeric value;
year(tau) = 2005 + (ord(tau)-1);

*	Include generic model parameterization

$include data

*	Set up model along the steady-state growth path 

parameter 
	esubt		Intertemporal elasticity of substitution         /0.5/,
	r		Baseline interest rate				 /0.05/,
	gr		Baseline growth rate				 /0.02/,
	depr		Depreciation rate				 /0.07/,
	kd0(kg)		Capital demands
	ks0		Capital stock
	pref(taup)	Reference path price level,
	qref(taup)	Reference path quantity level,
	k0		Base year capital stock,
	i0		Base year investment,
	pk0		Benchmark capital price
	rk0		Benchmark rental price ;

*	Move capital rents of fossil fuel production into sector-specific rents
*	and assign sector-specific capital stocks
sam("rent",ff) = sam("rent",ff) + sam("capital",ff);
sam("capital",ff) = 0;		

kd0("y") = -sam("capital","y");
kd0(xt)  = -xtelec("capital",xt);
ks0	 = sum(kg,kd0(kg));

qref(taup) = power(1+gr,	ord(taup)-1);
pref(taup) = power(1/(1+r),	ord(taup)-1);

pk0     = 1+r;
rk0     = r + depr;

*	Assign initial capital stock and investment consistent with steady state growth

k0(kg)	= kd0(kg)/rk0;
i0(kg)	= k0(kg)*(gr+depr);


sam("y","ra") = sam("y","ra") + sum(kg,i0(kg));
c0 = (-sam("y","ra")-sam("ele","ra")-sam("oil","ra") - sam("gas","ra"));  
display sam,i0;


parameter	eq(ff,tau)	Fossil-fuel rents
		en(n,tau)	Electricity inputs;

eq(ff,tau) = -sam("rent",ff)*qref(tau);
en(n,tau)  = nrsupply(n)*qref(tau);


*	For steady-state replication check there is no supply limit on active technologies
limit(t,tau) =  1e6*sum(xt, xtelec("ele",xt));

*	Definitions for renewable quota scenario
parameter	t_shr(tau)	Target share for green quota;
t_shr(tau) = 0;

*	Benchmark carbon emissions
carblim(tau) = sum(ff, carbon(ff))*qref(tau);


*	MPSGE formulation of the hybrid model
$ontext
$model:mps_dyn

$sectors:
	y(tau)			! Aggregate output
	ele(t,tau)		! Production levels for electricity by technology
	x(ff,tau)		! Fossil fuel supplies
	c(tau)			! Aggregate consumption (utility) formation	
	i(kg,tau)$tau(tau)	! Investment
	k(kg,tau)$tau(tau)	! Capital stock
	g$dd			! Public sector output
	w			! Intertemporal welfare
	lsp(tau)		! Labor supply
	cls(tau)		! Full consumption (including leisure)

$commodities:
	py(tau)		! Price of aggregate output
	pele(tau)	! Price of electricity
	pf(ff,tau)	! Price of fossil fuels
	pl(tau)		! Wage rate
	pls(tau)	! Leisure rate
	rk(kg,tau)	! Return to capital
	pk(kg,tau)	! Price of malleable capital for X (and NT elec)
	pq(ff,tau)	! Rent on fossil fuel resources
	pn(n,tau)	! Rent on natural resources
	pc(tau)		! Consumption (utility) price index	
	pkt		! Terminal capital stock
	plim(t,tau)$limit(t,tau)	! Administratively imposed limit
	pcarb(tau)$dd	! Carbon tax	
	pg$dd		! Public sector demand
	pw		! Intertemporal welfare price index
	pcls(tau)	! Price index of full consumption (including leisure)


$consumers:
	ra		! Representative agent
	govt$dd		! Government

	
$auxiliary:
	psi(tau)$card(quota)	! Subsidy associated with green quota
	phi_ls$dd		! Lump-sum recycling 	
	phi_tc$dd		! Consumption tax recycling	
	phi_tl$dd		! Labor tax recycling
	kt			! Terminal capital stock

*	Leisure

$prod:lsp(tau)
	o:pl(tau)	q:sam("labor","ra")
	i:pls(tau)	q:(sam("labor","ra")/(1+tl0$dd))	p:(1+tl0$dd)	a:govt$dd n:phi_tl$dd m:tl0$dd

*	Consumption:

$prod:c(tau)	s:esubc   ele(s):esubc_ele  nele(ele):esubc_nele
	o:pc(tau)	q:c0
	i:py(tau)	q:(-sam("y","ra")/(1+tc0$dd))	p:(1+tc0$dd) a:govt$dd n:phi_tc$dd  m:tc0$dd 
	i:pele(tau)	q:(-sam("ele","ra"))	ele:							 	
	i:pf(ff,tau)	q:(-sam(ff,"ra"))	nele:	


*	Aggregate output:

$prod:y(tau) s:esub	 kl(s):esub_kl	 ele(s):esub_ele nele(ele):esub_nele
	o:py(tau)	q:sam("y","y")
	i:pl(tau)	q:(-sam("labor","y"))	kl: 
	i:rk("y",tau)	q:kd0("y")		kl:
	i:pele(tau)	q:(-sam("ele","y"))	ele:
	i:pf(ff,tau)	q:(-sam(ff,"y"))	nele:

*	Extant electricity:	

$prod:ele(xt,tau)
	o:pele(tau)	q:1	a:ra   n:psi(tau)$quota(xt)	m:(-1)$quota(xt)  
	i:py(tau)	q:(-xtelec("y",xt)/xtelec("ele",xt))
	i:pf(ff,tau)	q:(-xtelec(ff,xt)/xtelec("ele",xt))
	i:pl(tau)	q:(-xtelec("labor",xt)/xtelec("ele",xt)) 	
	i:rk(xt,tau)	q:(kd0(xt)/xtelec("ele",xt))
	i:pn(n,tau)	q:(-xtelec(n,xt))	
	i:plim(xt,tau)$limit(xt,tau)	q:1



*	New vintage electricity:

$prod:ele(nt,tau)
	o:pele(tau)	q:1			a:ra   n:psi(tau)$quota(nt)	m:(-1)$quota(nt) 
	i:py(tau)	q:(-ntelec("y",nt))	
	i:rk(nt,tau)	q:(-ntelec("capital",nt))
	i:pl(tau)	q:(-ntelec("labor",nt))
	i:pn(n,tau)	q:(-ntelec(n,nt))
	i:plim(nt,tau)$limit(nt,tau)	q:1


$prod:x(ff,tau) s:0	r:esub_ff(ff)   xl(r):0
	o:pf(ff,tau)	q:sam(ff,ff)
	i:pcarb(tau)$dd	q:carbon(ff)
	i:pq(ff,tau)	q:(-sam("rent",ff))	r:
	i:py(tau)	q:(-sam("y",ff))	xl:
	i:pele(tau)	q:(-sam("ele",ff))	xl:							 	
	i:pf(f,tau)$(not sameas(f,ff))		q:(-sam(f,ff))		xl:
	i:pl(tau)	q:(-sam("labor",ff))	xl:


$prod:k(kg,tau)$tau(tau)
	o:rk(kg,tau)	q:(r+depr)
	o:pk(kg,tau+1)	q:(1-depr)
	o:pkt$tauL(tau)	q:(1-depr)
	i:pk(kg,tau)	q:1

$prod:i(kg,tau)$tau(tau)
	o:pk(kg,tau+1)	q:1
	o:pkt$tauL(tau)	q:1
	i:py(tau)	q:1

$prod:g$dd
	o:pg		q:(sum(tau,pref(tau)*qref(tau)*g0))
	i:py(tau)	q:(g0*qref(tau))

$constraint:psi(tau)$card(quota)
	sum(res(t), ele(t,tau)) =e= t_shr(tau) *sum(t, ele(t,tau));

$constraint:phi_ls$dd
	g =e= 1;

$constraint:phi_tc$dd
	g =e= 1;

$constraint:phi_tl$dd
	g =e= 1;


$prod:w	s:esubt		
	o:pw		q:(sum(tau,(qref(tau)*(c0 + ld0)*pref(tau)))) 
	i:pcls(tau)	q:(qref(tau)*(c0 + ld0))	p:pref(tau)

$prod:cls(tau) cl:esubl
	o:pcls(tau)		q:(c0 + ld0)
	i:pc(tau)	q:c0	p:pref(tau)	cl:
	i:pls(tau)	q:ld0	p:pref(tau)	cl:

$demand:ra    
	d:pw		q:(sum(tau,(qref(tau)*(c0+ld0)*pref(tau)))) 
	e:pk(kg,tau0)	q:(max(k0(kg),1e-6))
	e:pls(tau)	q:(qref(tau)*(sam("labor","ra")/(1+tl0$dd)+ld0))
	e:pq(ff,tau)	q:eq(ff,tau)
	e:pn(n,tau)	q:en(n,tau)
	e:pkt		q:(-1)		r:kt
	e:plim(t,tau)$limit(t,tau)	q:limit(t,tau)	
	e:pc(tau)$dd	q:(-qref(tau))	r:phi_ls


$demand:govt$dd
	d:pg		
	e:pc(tau)	q:qref(tau)	r:phi_ls
	e:pcarb(tau)	q:carblim(tau)


$constraint:kt
	sum(tauL(tau), sum(kg,I(kg,tau) - (1+gr)*I(kg,tau-1))) =e= 0;

$offtext
$sysinclude mpsgeset mps_dyn

*	In the base year new-vintage technologies are inactive
*	and the prices of backstop natural resources are zero
*	Extant technologies with capacity limits are assumed to
*	operate at the upper bound with a zero shadow value in the
*	base year


y.l(tau)	= qref(tau);
ele.l(t,tau)	= qref(tau);
x.l(ff,tau)	= qref(tau);
c.l(tau)	= qref(tau);
lsp.l(tau)	= qref(tau);
cls.l(tau)	= qref(tau);
i.l(kg,tau)	= qref(tau)*i0(kg);
k.l(kg,tau)	= qref(tau)*k0(kg);


pele.l(tau)	= pref(tau);
py.l(tau)	= pref(tau);
pf.l(ff,tau)	= pref(tau);
pl.l(tau)	= pref(tau);
pls.l(tau)	= pref(tau);
pcls.l(tau)	= pref(tau);
rk.l(kg,tau)	= pref(tau);
pk.l(kg,tau)	= pk0 * pref(tau);
pkt.l		= sum(tauL, pref(tauL));
pq.l(ff,tau)	= pref(tau);
pc.l(tau)	= pref(tau);
pw.l		= 1;
	
kt.l = sum(tauL(taup), qref(taup+1)) * sum(kg,k0(kg));

ele.l(nt,tau)   = 0;
pn.l(n,tau)     = 0;
plim.l(t,tau)	= 0;
ele.l(xt,tau)   = xtelec("ele",xt) * qref(tau);
pcarb.l(tau)    = 0;

 
*	Provisions fo public sector extension 
if(dd,
  g0  = 0.15 *(-sam("y","ra"));
  tc0 = g0/((-sam("y","ra")) - g0);
  phi_ls.lo	= -inf; phi_ls.up = +inf; 
  phi_ls.l	= -tl0*sam("labor","ra")/(1+tl0); 
  phi_tc.fx   	= 1;
  phi_tl.fx   	= 1; 
  pg.l   	= 1;
  g.l        	= 1;	
  );


*	Benchmark replication check
mps_dyn.workspace = 20;
mps_dyn.iterlim = 0;

$include mps_dyn.gen
solve mps_dyn using mcp;
display "The precision of the benchmark dataset is:", mps_dyn.objval;
abort$(ABS(mps_dyn.objval) gt 1e-4)"MPSGE model does not calibrate"; 


*	Relax iteration limit for counterfactual policy analysis
mps_dyn.iterlim      = 40000;

*	First compute off-the-steady-state baseline accounting for resource
*	limits on renewables and policy constraint on nuclear
*	- Hydro production is fixed at the baseyear level
*	- Other natural resource capacities are given in the base year as a fraction 
*	  of electricity output and thereafter can increase at the square of the 
*	  the economic growth rate up to a maximum of 30 % of reference (steady-state) 
*	  electricity production
*	- Nuclear power can not exceed the base-year level

en(n,tau)		= nrsupply(n) * min(qref(tau)**2, 0.3*qref(tau)*sam("ele","ele"));
en("water", tau) 	= nrsupply("water");
limit("nuclear",tau)	= xtelec("ele","nuclear");

$include mps_dyn.gen
solve mps_dyn using mcp;

*	Store BaU values

parameter	bau_stat	BaU statistics;

bau_stat(tau,t)		= ele.l(t,tau);
bau_stat(tau,"res")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat(tau,"carbon")	= sum(ff, x.l(ff,tau)*carbon(ff));
bau_stat(tau,"c")	= c.l(tau);
bau_stat("XXX","w")	= w.l;
bau_stat(tau,"i")	= sum(kg,i.l(kg,tau));
bau_stat(tau,"ele")	= sum(t,ele.l(t,tau));
bau_stat(tau,"y")	= y.l(tau);
bau_stat(tau,"rq")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));


*===========================================================================
*	Dynamic analysis of policy scenarios (as laid out in the paper)
*
*	(i)   gradual nuclear phase-out
*	(ii)  target quota for renewables (green quota)
*	(iii) carbon taxation (environmental tax reform)

*	Across all scenarios we assume that the policy regulation will take
*	place in the time interval given by [taustart, tauend]

scalar	taustart	Start year of regulatory policy change	/2010/,
	tauend		End year of regulatory policy change	/2030/;

set taulbl(tau)/2010,2020,2030,2040,2050,2060,2070,2080,2090,2100/;

alias(tau,ttau);


$setglobal labels taulbl
$setglobal domain tau
$setglobal gp_opt1 "set key below"
$setglobal gp_opt2 "set xlabel 'Time'"

parameter	delta		Changes in macroeconomic parameters (in percent),
		subsidy		Subsidy rate on electricity from renewables,		
		carbtax		Carbon permit price (carbon tax),
		rep_w		Percentage change in welfare (HEV);

*===========================================================================
*	Scenario 1: Gradual nuclear phase-out
*

limit("nuclear",tau) = 	bau_stat(tau,"nuclear");

loop(tau$(year(tau) GE taustart),
	limit("nuclear",tau)	= max(0, sum(ttau$(year(ttau) eq taustart), 
				     bau_stat(ttau,"nuclear"))*(1- (year(tau) - taustart)/(tauend-taustart)));
*       As nuclear capacity goes down to zero, shut down electricity production from nuclear
	ele.fx("nuclear",tau)$(not limit("nuclear",tau))	= 0;
);

$include mps_dyn.gen
solve mps_dyn using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);
rep_w("nuclear","XXX")  = round(100*(w.l/bau_stat("XXX","w") - 1),4);	 

$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot delta
delta(tau,"c") = 0; delta(tau,"i") = 0; delta(tau,"ele") = 0;

*	Re-initialize parameterization for subsequent scenarios
limit("nuclear",tau) = xtelec("ele","nuclear");
ele.lo("nuclear",tau) = 0; ele.up("nuclear",tau) = +inf;

*===========================================================================
*	Scenario 2: Green quotas
*
*	The share of power from renewable technologies in overall power production
*	is increased linearily by 10% relative to 2005 from "taustart" until "tauend" 
*	and then kept constant
t_shr(tau)			   = bau_stat("2005","rq");
t_shr(tau)$(year(tau) ge taustart) = min(1, sum(ttau$(year(ttau) eq taustart), 
					bau_stat("2005","rq")) + 10/100*(year(tau) - taustart)/(tauend-taustart));
t_shr(tau)$(year(tau) ge tauend)   = sum(ttau$(year(ttau) eq tauend), t_shr(ttau)); 


*	Impose green quota
quota(t)$res(t) = yes;

$include mps_dyn.gen
solve mps_dyn using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);
subsidy(tau)$(year(tau) ge taustart) = eps + 100*psi.l(tau);
rep_w("quota","XXX")  = round(100*(w.l/bau_stat("XXX","w") - 1),4);	 

$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot delta

$setglobal gp_opt3 "set ylabel 'Subsidy rate (% of electricity price)'"
$libinclude plot subsidy
delta(tau,"c") = 0; delta(tau,"i") = 0; delta(tau,"ele") = 0;
subsidy(tau) = 0;

*	Re-initialize parameterization for subsequent scenarios
quota(res) = no;

*===========================================================================
*	Scenario 3: Carbon taxation (double dividend)
*	Linear reduction of carbon emissions versus BaU by 20%

*	First re-calibrate base year (benchmark) accounting for public good extension
dd  = 1;	
g0  = 0.15 *(-sam("y","ra"));
tc0 = g0/((-sam("y","ra")) - g0);
phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
phi_tc.fx$dd = 1; phi_tl.fx$dd	= 1; pcarb.fx(tau) = 0;


*	Compute new BaU with public good provision and initial technology constraints
$include mps_dyn.gen
solve mps_dyn using mcp;

*	Store BaU values
bau_stat(tau,t)		= ele.l(t,tau);
bau_stat(tau,"res")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat(tau,"carbon")	= sum(ff, x.l(ff,tau)*carbon(ff));
bau_stat(tau,"c")	= c.l(tau);
bau_stat("XXX","w")	= w.l;
bau_stat(tau,"i")	= sum(kg,i.l(kg,tau));
bau_stat(tau,"ele")	= sum(t,ele.l(t,tau));
bau_stat(tau,"y")	= y.l(tau);
bau_stat(tau,"rq")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat("xxx","ls")	= phi_ls.l;
display bau_stat;


*	Linear reduction of carbon emissions by 20% between 
*	"taustart" and "tauend" holding vis-a-vis BaU values 
pcarb.lo(tau) = 0; pcarb.up(tau) = +inf; pcarb.l(tau) = 0;
carblim(tau)	=  bau_stat(tau,"carbon");
carblim(tau)$((year(tau) ge taustart) and (year(tau) le tauend))
		=  bau_stat(tau,"carbon")*(1- 0.2*(year(tau) - taustart)/(tauend-taustart));
carblim(tau)$(year(tau) ge tauend)	=  (1-0.2)*bau_stat(tau,"carbon"); 


parameter	cdelta	Percentage change in consumption;

*	Activate lump-sum transfer as recycling instrument
$include mps_dyn.gen
solve mps_dyn using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);

carbtax(tau,"ls") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","ls")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 
cdelta(tau,"ls")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);

*	Activate consumption tax as recycling instrument
	phi_tc.l = 1; phi_tc.lo = -0.99;  phi_tc.up = +inf; phi_ls.fx = bau_stat("XXX","ls"); phi_tl.fx = 1;
$include mps_dyn.gen
solve mps_dyn using mcp;

cdelta(tau,"tc")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
carbtax(tau,"tc") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","tc")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 


*	Activate wage tax as recycling instrument
	phi_tl.l = 1; phi_tl.lo = -0.99; phi_tl.up = +inf; phi_tc.fx = 1; phi_ls.fx = bau_stat("XXX","ls");
$include mps_dyn.gen
solve mps_dyn using mcp;

cdelta(tau,"tl")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
carbtax(tau,"tl") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","tl")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 


$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot cdelta

$setglobal gp_opt3 "set ylabel 'Carbon tax'"
$libinclude plot carbtax

display rep_w;
