$Title  Static TD/BU hybrid model - MPSGE Code

*	Include generic model parameterization
$include data

*	Initially there is only a policy constraint on the use of nuclear power
*	which is fixed at the base year level (operating at a shadow price of zero)
limit(t) = 1e+6 * elecgen("total");
limit("nuclear")	= xtelec("ele","nuclear");


set	quota(t)  Flag for technologies contributing to green quota;
scalar	t_shr	  Target share for green quota;

quota(t)= no; t_shr	= 0;

*	Benchmark carbon emissions
carblim = sum(ff, carbon(ff));

*	MPSGE formulation of the model
$ontext
$model:mps_static

$sectors:
	y	! Aggregate output
	ele(t)	! Production levels for electricity by technology
	x(ff)	! Fossil fuel supplies
	c	! Aggregate final consumption	
	lsp	! Labor supply
	g$dd	! Public good provision
	w	! Full consumption (utility) including leisure	

$commodities:
	py	! Price of aggregate output
	pele	! Price of electricity
	pf(ff)	! Price of fossil fuels
	pl	! Wage rate
	pls	! Leisure rate
	pk	! Price of malleable capital for X (and NT elec)
	pq(ff)	! Rent on fossil fuel resources
	pn(n)	! Rent on natural resources
	pc	! Consumption (utility) price index	
	pw	! Price of full consumption including leisure
	plim(t)$limit(t) ! Shadow price on electricity expansion
	pkx(t)$kx(t)	 ! Price of capital to extant technologies
	pg$dd		 ! Price of public consumption
	pcarb$dd	 ! Carbon tax rate
	

$consumers:
	ra	! Representative agent
	govt$dd	! Government


$auxiliary:
	psi$card(quota)  ! Uniform subsidy rate on renewable energy	
	phi_ls$dd	 ! Lump-sum recycling of carbon tax revenues 	
	phi_tc$dd	 ! Consumption tax recycling of carbon tax revenues	
	phi_tl$dd	 ! Labor tax recycle of carbon tax revenue

*	Aggregate output:

$prod:y s:esub	 kl(s):esub_kl	 ele(s):esub_ele nele(ele):esub_nele
	o:py	 q:sam("y","y")
	i:pl	 q:(-sam("labor","y"))   kl: 
	i:pk	 q:(-sam("capital","y")) kl:
	i:pele	 q:(-sam("ele","y"))	 ele:
	i:pf(ff) q:(-sam(ff,"y"))	 nele:


*	Extant electricity:	

$prod:ele(xt)
	o:pele		q:1	a:ra   n:psi$quota(xt)	m:(-1)$quota(xt)  
	i:py		q:(-xtelec("y",xt)/xtelec("ele",xt))
	i:pf(ff)	q:(-xtelec(ff,xt)/xtelec("ele",xt))
	i:pl		q:(-xtelec("labor",xt)/xtelec("ele",xt)) 	
	i:pkx(xt)	q:(kx(xt)/xtelec("ele",xt))
	i:pk		q:(-(xtelec("capital",xt)+kx(xt))/xtelec("ele",xt))
	i:pn(n)		q:(-xtelec(n,xt))	
	i:plim(xt)$limit(xt)  q:1

*	New vintage electricity:

$prod:ele(nt)
	o:pele		q:1	a:ra   n:psi$quota(nt)	m:(-1)$quota(nt) 
	i:py		q:(-ntelec("y",nt))
	i:pk		q:(-ntelec("capital",nt))
	i:pl		q:(-ntelec("labor",nt))
	i:pn(n)		q:(-ntelec(n,nt))
	i:plim(nt)$limit(nt)  q:1

*	Fossil fuel production:

$prod:x(ff) s:0	r:esub_ff(ff)   xl(r):0
	o:pf(ff)	q:sam(ff,ff)
	i:pcarb$dd	q:carbon(ff)
	i:py		q:(-sam("y",ff))	xl:
	i:pele		q:(-sam("ele",ff))	xl:							 	
	i:pf(f)$(not sameas(f,ff))	q:(-sam(f,ff))		xl:
	i:pl		q:(-sam("labor",ff))	xl:
	i:pk		q:(-sam("capital",ff))	xl:
	i:pq(ff)	q:(-sam("rent",ff))	r:

*	Final consumption:

$prod:c	s:esubc   ele(s):esubc_ele  nele(ele):esubc_nele
	o:pc		q:c0
	i:py		q:(-sam("y","ra")/(1+tc0$dd))	p:(1+tc0$dd) a:govt$dd n:phi_tc$dd  m:tc0$dd 
	i:pele		q:(-sam("ele","ra"))	ele:							 	
	i:pf(ff)	q:(-sam(ff,"ra"))	nele:	

*	Public good provision:

$prod:g$dd
	O:pg	q:g0
	i:py	q:g0

$prod:lsp
	o:pl	q:sam("labor","ra")
	i:pls	q:(sam("labor","ra")/(1+tl0$dd))	p:(1+tl0$dd)	a:govt$dd n:phi_tl$dd m:tl0$dd

$prod:w	s:esubl
	o:pw	q:(c0 + ld0)
	i:pc	q:c0
	i:pls	q:ld0


*	Income - expenditure of representative household:

$demand:ra   
	d:pw			q:(c0 + ld0)
	e:pk			q:(-sam("capital","y")+ sum(ff,-sam("capital",ff)) + sum(xt,(-xtelec("capital",xt)) - kx(xt)))
	e:pkx(xt)		q:kx(xt)
	e:pls			q:(sam("labor","ra")/(1+tl0$dd)+ld0)
	e:pq(ff)		q:(-sam("rent",ff))
	e:pn(n)			q:nrsupply(n)
	e:plim(t)$limit(t)	q:limit(t)
	e:pc$dd			q:(-1)		r:phi_ls$dd

$demand:govt$dd
	d:pg		q:g0
	e:pc		q:1		r:phi_ls
	e:pcarb$dd	q:carblim

*	Green quota constraint:

$constraint:psi$card(quota)
	sum(res(t), ele(t)) =e= t_shr*sum(t, ele(t));

$constraint:phi_tl$dd
	g =e= 1;

$constraint:phi_ls$dd
	g =e= 1;

$constraint:phi_tc$dd
	g=e= 1;

$offtext
$sysinclude mpsgeset mps_static

*	In the base year new-vintage technologies are inactive
*	and the prices of backstop natural resources are zero
*	Extant technologies with capacity limits are assumed to
*	operate at the upper bound with a zero shadow value in the
*	base year

ele.l(nt)  = 0;
pn.l(n)	   = 0;
plim.l(t)  = 0;

ele.l(xt)  = xtelec("ele",xt);

*	Benchmark replication check
mps_static.iterlim = 0;
$include mps_static.gen
solve mps_static using mcp;
display "The precision of the benchmark dataset is:", mps_static.objval;
abort$(ABS(mps_static.objval) gt 1e-4)"MPSGE model does not calibrate"; 

*	Relax iteration limit for counterfactual policy analysis
mps_static.iterlim      = 40000;



*===========================================================================
*	Static analysis of policy scenarios (as laid out in the paper)

*	Scenario 1:	Gradual nuclear phase-out
*	Scenario 2:	Target quota for renewables (green quota)
*	Scenario 3:	Carbon taxation (environmental tax reform)


*
*	Define report parameters
parameter
	ev	 Equivalent variation in income
	supply	 Electricity supply by technology
	carbtax	 Carbon permit price
	subsidy	 Subsidy rate on electricity from renewables;		

*---------  Scenario 1: Gradual nuclear phase-out --------------------------

set	nsc  Nuclear phase scenarios / 0, 25, 50, 75, 100/;


*	Sub-scenario: No supply limits on fossil fuel based power generation
loop(nsc,

*	Assign available capacity for nuclear power 
	limit("nuclear") = (1 - (ord(nsc)-1)/(card(nsc)-1))*xtelec("ele","nuclear");
*	If nuclear capacity is set to zero, assure complete nuclear phase out	
	if ((not limit("nuclear")),
	   ele.fx("nuclear") = 0;
	   );
$include mps_static.gen
	solve mps_static using mcp;
	supply(nsc,t)	    = ele.l(t);
	ev(nsc,"fossil")    = round(100 * (w.l-1), 3) + eps;
);

display ev, supply;

*	Sub-scenario: Base-year limits on fossil fuel based power generation
ele.up("nuclear")=+inf;
limit("coal") = xtelec("ele","coal");
limit("gas")  = xtelec("ele","gas");
limit("oil")  = xtelec("ele","oil");

loop(nsc,

*	Assign available capacity for nuclear power 
	limit("nuclear") = (1 - (ord(nsc)-1)/(card(nsc)-1))*xtelec("ele","nuclear");

*	If nuclear capacity is set to zero, assure complete nuclear phase out	
	if ((not limit("nuclear")),  ele.fx("nuclear") = 0;  );

$include mps_static.gen
	solve mps_static using mcp;
	ev(nsc,"nofossil")  = 100 * (w.l-1) + eps;
	supply(nsc,t)	    = ele.l(t);

);

*	See if GNUPLOT is installed:

$if not exist '%gams.sysdir%wgnuplot.exe' $exit
$if not exist '%gams.ldir%plot.gms'     $exit

*	If GNUPLOT is installed  -- generate figures at the terminal:

$setglobal labels nsc
$setglobal gp_opt0 "set data style linespoints"
$setglobal gp_opt1 "set key below"

$setglobal gp_opt2 "set title 'Technology shifts'"
$setglobal gp_opt3 "set xlabel 'Nuclear capacity reduction (% vis--vis BaU)'"
$setglobal gp_opt4 "set ylabel 'Activity level of technologies'"
$libinclude plot supply
supply(nsc,t) = 0; 

$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set ylabel 'Equivalent variation in income (%)'"
$libinclude plot ev
ev(nsc,"fossil") = 0; ev(nsc,"nofossil") = 0;

*	Re-initialize parameterization for subsequent scenarios

ele.up("nuclear")= +inf;
limit(t) = 1e+6 * elecgen("total");
limit("nuclear")	= xtelec("ele","nuclear");

*------------------------ Scenario 2: Green quotas -------------------------

*	Note:	We start from the benchmark share of renewables in power
*		generation and then increase this share iteratively by 2% 
*		in five steps (towards a total increase of 10%).

scalar	bmk_shr		Benchmark renewable share in electricity generation;
bmk_shr = sum(t$res(t), xtelec("ele",t))/sum(t, xtelec("ele",t));

set	qsc  Iterative steps for green quota scenario / 1*6 /;

quota(res) = yes;

quota(res) = yes; psi.lo = -inf; psi.up = +inf; psi.l = 0;  
y.l = 1; ele.l(xt)= xtelec("ele",xt);  x.l(ff) = 1; c.l = 1; w.l = 1; lsp.l = 1; 
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pls.l = 1; pk.l = 1; pq.l(ff) = 1;                                 
pkx.l(t)$kx(t) = 1; plim.l(t) = 0; pw.l = 1; pn.l(n) = 0; pc.l = 1; 

*	First calculate long-run impact with malleable capital of extant technologies
kx(xt) = 0;

loop(qsc,

*	Assign target shares for renewables in electricity production

	t_shr  = min(1, (bmk_shr + 10/100* (ord(qsc)-1)/(card(qsc)-1)));

$include mps_static.gen
	solve mps_static using mcp;

	supply(qsc,t)		= ele.l(t)  + eps;
	ev(qsc,"long run")	= 100 * (w.l-1) + eps;
	subsidy(qsc,"long run")	= 100*psi.l + eps;
);




*	Then calculate short-run impact with fixed capital of extant technologies
kx(xt) = -xtelec("capital",xt); pkx.up(xt)$kx(xt) = +inf; pkx.l(xt)$kx(xt) = 1;  
psi.lo = 0; psi.up = +inf; psi.l = 0;  
y.l = 1; ele.l(xt)= xtelec("ele",xt);  x.l(ff) = 1; c.l = 1; w.l = 1; lsp.l = 1; 
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pls.l = 1; pk.l = 1; pq.l(ff) = 1;                                 
pkx.l(t)$kx(t) = 1; plim.l(t) = 0; pw.l = 1; pn.l(n) = 0; pc.l = 1; 


loop(qsc,

*	Assign target shares for renewables in electricity production

	t_shr  = min(1, (bmk_shr + 10/100* (ord(qsc)-1)/(card(qsc)-1)));

$include mps_static.gen
	solve mps_static using mcp;

	supply(qsc,t)		 = ele.l(t)  + eps;
	ev(qsc,"short run")      = 100 * (w.l-1) + eps;
	subsidy(qsc,"short run") = 100*psi.l + eps;

);

$setglobal labels qsc
$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set xlabel 'Green quota in % of overall electricity supply'"
$setglobal gp_opt4 "set ylabel 'Equivalent variation in income (%)'"
$setglobal gp_opt5 "set xtics ('18' 1, '20' 2, '22' 3, '24' 4, '26' 5, '28' 6)"

$libinclude plot ev
ev(qsc,"long run") = 0; ev(qsc,"short run") = 0; 

$setglobal gp_opt2 "set title 'Implicit Subsidy'"
$Setglobal gp_opt4 "set ylabel 'Subsidy rate (% of electricity price)'"
$libinclude plot subsidy
subsidy(qsc,"long run")	= 0; subsidy(qsc,"short run") = 0;

*	Re-initialize parameterization for subsequent scenarios
quota(t) = no; kx(t) = 0;


*------------ Scenario 3: Carbon taxation (Environmental tax reform) -------

*	First re-calibrate benchmark to public good setting
mps_static.iterlim      = 0;
dd  = 1; 
*	Set public consumption at 15% of private consumption (of Y) 
g0  = 0.15 *(-sam("Y","ra")); 
tc0 = g0/((-sam("Y","ra")) - g0);
display tc0;


*	Initially, we assume that lump-sum transfers are active
*	as the equal-yield instrument
phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
phi_tc.fx = 1; phi_tl.fx  = 1; 


*	Benchmark replication check for the model with public good extension
*	Initialize activities and prices

y.l = 1; ele.l(xt)= xtelec("ele",xt);  ele.l(nt) = 0; x.l(ff) = 1; c.l = 1;	
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pk.l = 1; pq.l(ff) = 1;					
pkx.l(t) = 1; plim.l(t)$limit(t) = 0; pc.l = 1; pn.l(n) = 0; 
pcarb.l = 0; pg.l = 1; lsp.l = 1; pls.l = 1; w.l = 1; pw.l = 1;

mps_static.iterlim = 0;
$include mps_static.gen
solve mps_static using mcp;

display "The precision of the re-specified benchmark dataset is:", mps_static.objval;
abort$(ABS(mps_static.objval) gt 1e-4)"MPSGE model does not calibrate"; 

*	Relax iteration limit
mps_static.iterlim = 4000;

*	Specification of carbon tax scenarios based on exogenous emission reduction targets
set	csc  Carbon abatement scenarios scenarios / 0, 5, 10, 15, 20/;

loop(csc,
*	Assign carbon emission limit
	carblim = (1 - 0.2*(ord(csc)-1)/(card(csc)-1))*sum(ff, carbon(ff));

*	Activate lump-sum transfer as recycling instrument
	phi_ls.l  = 0; phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
	phi_tc.fx = 1; phi_tl.fx = 1;
$include mps_static.gen
	solve mps_static using mcp;
	ev(csc,"ls") 	  =  round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"ls") =  round((1/multi)*pcarb.l/pc.l, 0) + eps;

*	Activate consumption tax as recycling instrument
	phi_tc.l = 1; phi_tc.lo = -0.99;  phi_tc.up = +inf; phi_ls.fx = -tl0*sam("labor","ra")/(1+tl0); phi_tl.fx = 1;
$include mps_static.gen
	solve mps_static using mcp;
	ev(csc,"tc")	  = round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"tc") = round((1/multi)*pcarb.l/pc.l, 0) + eps;

*	Activate wage tax as recycling instrument
	phi_tl.l = 1; phi_tl.lo = -0.99; phi_tl.up = +inf; phi_tc.fx = 1; phi_ls.fx = -tl0*sam("labor","ra")/(1+tl0);
$include mps_static.gen
	solve mps_static using mcp;
	ev(csc,"tl") 	  = round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"tl") = round((1/multi)*pcarb.l/pc.l, 0) + eps;
);

$setglobal labels csc
$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set xlabel 'Carbon emission reduction (in % vis--vis base year)'"

$setglobal gp_opt4 "set ylabel 'Equivalent variation in income (%)'"
$libinclude plot ev
ev(csc,"tl") = 0; ev(csc,"tc") = 0; ev(csc,"ls") = 0;

$setglobal gp_opt2 "set title 'Carbon Tax Rate'"
$setglobal gp_opt4 "set ylabel 'Carbon tax in $ per ton of C'"
$libinclude plot carbtax
carbtax(csc,"tl") = 0; carbtax(csc,"tc") = 0; carbtax(csc,"ls") = 0;

